﻿using WarmlyShip.CollectionGenericObjects;
using WarmlyShip.Drawnings;

namespace WarmlyShip.CollectionGenericObjects;


public abstract class AbstractCompany
{

    protected readonly int _placeSizeWidth = 150;

    protected readonly int _placeSizeHeight = 85;


    protected readonly int _pictureWidth;
    
    
    protected readonly int _pictureHeight;

   
    protected ICollectionGenericObjects<DrawningShip> _collection = null;

  
    private int GetMaxCount => _pictureWidth * _pictureHeight / ((_placeSizeWidth + _placeSizeWidth /2) * _placeSizeHeight) - 1;

    
    public AbstractCompany(int picWidth,  int picHeight, ICollectionGenericObjects<DrawningShip> collection)
    {
        _pictureWidth = picWidth;
        _pictureHeight = picHeight;
        _collection = collection;
        _collection.MaxCount = GetMaxCount;
    }


    public static int operator +(AbstractCompany company, DrawningShip ship)
    {
        return company._collection.Insert(ship);
    }

    public static DrawningShip operator -(AbstractCompany company, int position)
    {
        return company._collection.Remove(position);
    }


    public DrawningShip? GetRandomObject()
    {
        Random rnd = new();
        return _collection?.Get(rnd.Next(GetMaxCount));
    }

  
    public Bitmap? Show()
    {
        Bitmap bitmap = new(_pictureWidth, _pictureHeight);
        Graphics graphics = Graphics.FromImage(bitmap);
        DrawBackgroud(graphics);

        SetObjectPosition();
        for (int i = 0; i < (_collection?.Count ?? 0); i++)
        {
            DrawningShip? obj = _collection?.Get(i);
            obj?.DrawTransport(graphics);
        }

        return bitmap;
    }

    protected abstract void DrawBackgroud(Graphics g);


    protected abstract void SetObjectPosition();
}
