﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using WarmlyShip.Entities;

namespace WarmlyShip.Drawnings;

public class DrawningShip
{


    public EntityShip? EntityShip { get; protected set; }


    private int? _pictureWidth;

    private int? _pictureHeight;

    protected int? _startPosX;

    protected int? _startPosY;

    private readonly int _drawningWarmlyShipWidth = 150;

    private readonly int _drawningWarmlyShipHeight = 80;

    public int? GetPosX => _startPosX;

    
    public int? GetPosY => _startPosY;

   
    public int GetWidth => _drawningWarmlyShipWidth;

    public int GetHeight => _drawningWarmlyShipHeight;


    private DrawningShip() {
        _pictureHeight = null;
        _pictureWidth = null;
        _startPosX = null;
        _startPosY = null;
}
    public DrawningShip (int speed, double weight, Color bodyColor) : this()
    {
        EntityShip = new EntityShip(speed, weight, bodyColor);
        
    }

    public DrawningShip(EntityShip ship) : this()
    {
        EntityShip = new EntityShip(ship.Speed, ship.Weight, ship.BodyColor);

    }

    protected DrawningShip(int drawningWarmlyShipWidth, int drawningWarmlyShipHeigh) : this()
{
        _drawningWarmlyShipWidth = drawningWarmlyShipWidth;
        _drawningWarmlyShipHeight= drawningWarmlyShipHeigh;
        
    }

    public bool SetPictureSize(int width, int height)
    {
        if (width > _drawningWarmlyShipWidth && height > _drawningWarmlyShipHeight)
        {
            _pictureWidth = width;
            _pictureHeight = height;
            if (_startPosX != null && _startPosY != null)
            {
                if (_startPosX.Value + _drawningWarmlyShipWidth > _pictureWidth)
                {
                    _startPosX = _pictureWidth - _drawningWarmlyShipWidth;
                }
                if (_startPosY.Value + _drawningWarmlyShipHeight > _pictureHeight)
                {
                    _startPosY = _pictureHeight - _drawningWarmlyShipHeight;
                }
            }
            return true;
        }
        return false;

    }
    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }
        else
        {
            _startPosX = x;
            _startPosY = y;

            if (_startPosX < 0) _startPosX = 0;
            if (_startPosY < 0) _startPosY = 0;

            if (_startPosX + _drawningWarmlyShipWidth > _pictureWidth.Value)
            {
                _startPosX = _pictureWidth.Value - _drawningWarmlyShipWidth;
            }
            if (_startPosY + _drawningWarmlyShipHeight > _pictureHeight.Value)
            {
                _startPosY = _pictureHeight.Value - _drawningWarmlyShipHeight;
            }
        }

    }
    public bool MoveTransport(DirectionType direction)
    {
        if (EntityShip == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return false;
        }

        switch (direction)
        {
            case DirectionType.Left:
                if (_startPosX.Value - EntityShip.Step > 0)
                {
                    _startPosX -= (int)EntityShip.Step;
                }
                return true;

            case DirectionType.Up:
                if (_startPosY.Value - EntityShip.Step > 0)
                {
                    _startPosY -= (int)EntityShip.Step;
                }
                return true;

            case DirectionType.Right:
                if (_startPosX.Value + _drawningWarmlyShipWidth + EntityShip.Step < _pictureWidth)
                {
                    _startPosX += (int)EntityShip.Step;
                }
                return true;

            case DirectionType.Down:
                if (_startPosY.Value + _drawningWarmlyShipHeight + EntityShip.Step < _pictureHeight)
                {
                    _startPosY += (int)EntityShip.Step;
                }
                return true;

            default: return false;
        }
    }

    public virtual void DrawTransport(Graphics g)
    {
        if (EntityShip == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }

  
        // нарисовать корабль
        Pen pen = new(Color.Black, 2);
        Brush brMain = new SolidBrush(EntityShip.BodyColor);
        // трапеция
        Point[] points = new Point[]
   {

        new Point(_startPosX.Value, _startPosY.Value+50),
        new Point(_startPosX.Value + 150, _startPosY.Value+ 50),
        new Point(_startPosX.Value + 120, _startPosY.Value + 80),
        new Point(_startPosX.Value + 30, _startPosY.Value + 80)

   };
        g.FillPolygon(brMain, points);
        g.DrawPolygon(pen, points);

        //палуба
        g.FillRectangle(brMain, _startPosX.Value + 25, _startPosY.Value + 30, 100, 20);
        g.DrawRectangle(pen, _startPosX.Value + 25, _startPosY.Value + 30, 100, 20);

        //якорь
        g.DrawLine(pen, _startPosX.Value + 30, _startPosY.Value + 53, _startPosX.Value + 30, _startPosY.Value + 70);
        g.DrawLine(pen, _startPosX.Value + 23, _startPosY.Value + 70, _startPosX.Value + 37, _startPosY.Value + 70);
        g.DrawLine(pen, _startPosX.Value + 23, _startPosY.Value + 60, _startPosX.Value + 37, _startPosY.Value + 60);

        

       
    }

}