﻿using WarmlyShip.CollectionGenericObjects;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using WarmlyShip.Drawnings;

namespace WarmlyShip
{
    public partial class FormShipCollection : Form
    {
        public FormShipCollection()
        {
            InitializeComponent();
            _storageCollection = new();
        }
        private AbstractCompany? _company = null;
        private readonly StorageCollection<DrawningShip> _storageCollection;




        private void ComboBoxSelectorCompany_SelectedIndexChanged(Object sender, EventArgs e)
        {
            panelCompanyTools.Enabled = false;
        }




        private void ButtonAddShip_Click(object sender, EventArgs e)
        {
            FormShipConfig form = new();

            form.Show();
            form.AddEvent(SetShip);
        }


        private void SetShip(DrawningShip ship)
        {
            if (_company == null || ship == null)
            {
                return;
            }

            if (_company + ship != -1)
            {
                MessageBox.Show("Объект добавлен");
                pictureBox.Image = _company.Show();
            }
            else
            {
                MessageBox.Show("Не удалось добавить объект");
            }
        }


        private void ButtonRemoveShip_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(maskedTextBoxPosition.Text) || _company == null)
            {
                return;
            }

            if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
            {
                return;
            }

            int pos = Convert.ToInt32(maskedTextBoxPosition.Text);
            if (_company - pos != null)
            {
                MessageBox.Show("Объект удален");
                pictureBox.Image = _company.Show();
            }
            else
            {
                MessageBox.Show("Не удалось удалить объект");
            }
        }

        private void ButtonGoToCheck_Click(object sender, EventArgs e)
        {
            if (_company == null)
            {
                return;
            }

            DrawningShip? ship = null;
            int counter = 100;
            while (ship == null)
            {
                ship = _company.GetRandomObject();
                counter--;
                if (counter <= 0)
                {
                    break;
                }
            }

            if (ship == null)
            {
                return;
            }

            FormShips form = new()
            {
                SetShip = ship
            };
            form.ShowDialog();
        }

        private void ButtonRefresh_Click(object sender, EventArgs e)
        {
            if (_company == null)
            {
                return;
            }

            pictureBox.Image = _company.Show();
        }


        private void ButtonCollectionAdd_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxCollectionName.Text) || (!radioButtonList.Checked && !radioButtonMssive.Checked))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            CollectionType collectionType = CollectionType.None;
            if (radioButtonMssive.Checked)
            {
                collectionType = CollectionType.Massive;
            }
            else if (radioButtonList.Checked)
            {
                collectionType = CollectionType.List;
            }

            _storageCollection.AddCollection(textBoxCollectionName.Text, collectionType);
            RefreshListBoxItems();
        }

        private void ButtonCollectionDel_Click(object sender, EventArgs e)
        {
            if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
            {
                MessageBox.Show("Коллекция не выбрана");
                return;
            }

            if (MessageBox.Show("Удалить коллекцию?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }

            _storageCollection.DelCollection(listBoxCollection.SelectedItem.ToString());
            RefreshListBoxItems();

        }

        private void ButtonCreateCompany_Click(object sender, EventArgs e)
        {
            if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItems == null)
            {
                MessageBox.Show("Коллекция не выбрана");
                return;
            }

            ICollectionGenericObjects<DrawningShip>? collection = _storageCollection[listBoxCollection.SelectedItem?.ToString() ?? string.Empty];
            if (collection == null)
            {
                MessageBox.Show("Коллекция не проинициализирована");
                return;
            }

            switch (comboBoxSelectorCompany.Text)
            {
                case "Хранилище":
                    _company = new PortForShips(pictureBox.Width, pictureBox.Height, collection);
                    break;
            }

            panelCompanyTools.Enabled = true;
            RefreshListBoxItems();
        }
        private void RefreshListBoxItems()
        {
            listBoxCollection.Items.Clear();
            for (int i = 0; i < _storageCollection.Keys?.Count; ++i)
            {
                string? colName = _storageCollection.Keys?[i];
                if (!string.IsNullOrEmpty(colName))
                {
                    listBoxCollection.Items.Add(colName);
                }
            }
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                if (_storageCollection.SaveData(saveFileDialog.FileName))
                {
                    MessageBox.Show("Сохранение прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                else
                {
                    MessageBox.Show("Не сохранилось", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void loadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                if (_storageCollection.LoadData(openFileDialog.FileName))
                {
                    MessageBox.Show("Загрузка прошла успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                else
                {
                    MessageBox.Show("Не загрузилось", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            RefreshListBoxItems();
        }
    }
    
}
