﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using WarmlyShip.Drawnings;
using WarmlyShip.Entities;

namespace WarmlyShip
{
    public partial class FormShipConfig : Form
    {

        private DrawningShip? _ship;
        private event Action<DrawningShip>? ShipDelegate;
        public FormShipConfig()
        {
            InitializeComponent();
            panelRed.MouseDown += panel_MouseDown;
            panelGreen.MouseDown += panel_MouseDown;
            panelBlue.MouseDown += panel_MouseDown;
            panelYellow.MouseDown += panel_MouseDown;
            panelWhite.MouseDown += panel_MouseDown;
            panelGrey.MouseDown += panel_MouseDown;
            panelBlack.MouseDown += panel_MouseDown;
            panelPurple.MouseDown += panel_MouseDown;

            buttonCancel.Click += (sender, e) => Close();

        }

        public void AddEvent(Action<DrawningShip> shipDelegate)
        {
            ShipDelegate += shipDelegate;
        }

        private void drawObject()
        {
            Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
            Graphics g = Graphics.FromImage(bmp);
            _ship?.SetPictureSize(pictureBoxObject.Width, pictureBoxObject.Height);
            _ship?.SetPosition(5, 5);
            _ship?.DrawTransport(g);
            pictureBoxObject.Image = bmp;
        }



        private void labelObject_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Label)?.DoDragDrop((sender as Label)?.Name ?? string.Empty, DragDropEffects.Move | DragDropEffects.Copy);
        }

        private void panelObject_DragEnter(object sender, DragEventArgs e)
        {
            e.Effect = e.Data?.GetDataPresent(DataFormats.Text) ?? false ? DragDropEffects.Copy : DragDropEffects.None;
        }

        private void panelObject_DragDrop(object sender, DragEventArgs e)
        {
            switch (e.Data?.GetData(DataFormats.Text)?.ToString())
            {
                case "labelSimpleObject":
                    _ship = new DrawningShip((int)numericUpDownSpeed.Value, (int)numericUpDownWeight.Value, Color.White);
                    break;
                case "labelModifiedObject":
                    _ship = new DrawningWarmlyShip((int)numericUpDownSpeed.Value, (int)numericUpDownWeight.Value, Color.White,
                        Color.Black, checkBoxFuelHole.Checked, checkBoxPipes.Checked);
                    break;
            }
            drawObject();
        }
        private void panel_MouseDown(object? sender, MouseEventArgs e)
        {
            (sender as Control)?.DoDragDrop((sender as Control).BackColor, DragDropEffects.Move | DragDropEffects.Copy);
        }

        private void labelBaseColor_DragDrop(object sender, DragEventArgs e)
        {
            if (_ship != null)
            {
                _ship.EntityShip.setBodyColor((Color)e.Data.GetData(typeof(Color)));
                drawObject();
            }
        }

        private void labelBaseColor_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(typeof(Color)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void labelSeckondColor_DragDrop(object sender, DragEventArgs e)
        {
            if (_ship.EntityShip is EntityWarmlyShip _warmlyShip)
            {
                _warmlyShip.setSeckondColor((Color)e.Data.GetData(typeof(Color)));
            }
            drawObject();
        }

        private void labelSeckondColor_DragEnter(object sender, DragEventArgs e)
        {
            if (_ship is DrawningWarmlyShip)
            {
                if (e.Data.GetDataPresent(typeof(Color)))
                {
                    e.Effect = DragDropEffects.Copy;
                }
                else
                {
                    e.Effect = DragDropEffects.None;
                }
            }
        }


        private void buttonAdd_Click(object sender, EventArgs e)
        {
            if (_ship != null)
            {
                ShipDelegate?.Invoke(_ship);
                Close();
            }

        }
    }
}
