﻿using WarmlyShip.MovementStrategy;
using WarmlyShip.Drawnings;



namespace ProjectShip.MovementStrategy;

public class MoveableShip : IMoveableObject
{
    private readonly DrawningShip? _drawningShip = null;

    public MoveableShip(DrawningShip ship)
    {
        _drawningShip = ship;
    }

    public ObjectParameters? GetObjectPosition
    {
        get
        {
            if (_drawningShip == null || _drawningShip.EntityShip == null || !_drawningShip.GetPosX.HasValue || !_drawningShip.GetPosY.HasValue)
            {
                return null;
            }

            return new ObjectParameters(_drawningShip.GetPosX.Value, _drawningShip.GetPosY.Value, _drawningShip.GetWidth, _drawningShip.GetHeight);
        }
    }

    public int GetStep => (int)(_drawningShip?.EntityShip?.Step ?? 0);

    public bool TryMoveObject(MovementDirection direction)
    {
        if(_drawningShip == null || _drawningShip.EntityShip == null)
        {
            return false;
        }

        return _drawningShip.MoveTransport(GetDirectionType(direction));
    }

    private static DirectionType GetDirectionType(MovementDirection direction)
    {
        return direction switch
        {
            MovementDirection.Left => DirectionType.Left,
            MovementDirection.Right => DirectionType.Right,
            MovementDirection.Up => DirectionType.Up,
            MovementDirection.Down => DirectionType.Down,
            _ => DirectionType.Unknow,
        };
    }
}
